// README
//  
// Javascript functions to handle AJAX variables updating in HTML pages
//

function getTextUIElementForVar(varname) {
  // since our divs have the same id as the name of the corresponding 
  // vars, we use it as a simple binding property and search for it 
  // (escaping with double backslash CSS notation characters, which
  // are problematic within the context of a jQuery selector)
  return $('#' + varname.replace( /(:|\.|\[|\])/g, "\\$1" ));
}
        
function getImageUIElementForVar(varname) {
  // since our images have the same id as the name of the corresponding 
  // vars we use it as a simple binding property and search for it 
  return $('img[id="' + varname + '"]');
}

function getInputUIElementForVar(varname) {
  // input elements are identified by their name (there can be more
  // than one input element with the same name bu different type)
  return $('[name="' + varname + '"]');
}

function findDeviceVariable(varname) {
  for( var i = 0; i < mDeviceVariables.length; i++ ) {
    var variable = mDeviceVariables[i];
    if ( variable.getName() === varname ) {
      return variable;
    }
  }
  return null;
}

function deviceEngineResultCallback(result, variables) {
  mDeviceVariables = variables;
  if( result && variables != null ) {
    for( var i = 0; i < variables.length; i++ ) {
      updateTextFromRemote(variables[i]);
      updateInputFromRemote(variables[i]);
      updateImageFromRemote(variables[i]);
    }
  } else {
    for ( var v in mVariablesUIModel ) {
      disableUIElementsByVarName(v);
    }
  }
  startRefresh(mRefreshInterval);
}

function disableUIElementsByVarName(varname) {
  var valueElement = getTextUIElementForVar(varname);
  if( typeof(valueElement) !== 'undefined' ) {
    valueElement.text('---');
  }
  var inputElement = getInputUIElementForVar(varname);
  inputElement.each(function() {
    var currObj = $(this);
    currObj.prop('disabled', true);
  });
}

function updateTextFromRemote(remoteVar) {
 var valueElement = getTextUIElementForVar(remoteVar.getName());
 if( typeof(valueElement) !== 'undefined' ) {
   var valueContent;
   if( remoteVar.getStatus() === 'OK' ) {
     var um = mVariablesUIModel[remoteVar.getName()].um || '';
     var txt = mVariablesUIModel[remoteVar.getName()].txt || '';
     valueContent = txt + formatRemoteValue(remoteVar) + um;
   } else {
     valueContent = remoteVar.getStatus();
   }
   valueElement.text(valueContent);
   valueElement.html(valueContent);
 }
}
        
function updateImageFromRemote(remoteVar) {
  var valueElement = getImageUIElementForVar(remoteVar.getName());
  if( typeof(valueElement) !== 'undefined' ) {
    var valueContent;
    if( remoteVar.getStatus() === 'OK' ) {
      valueContent = '../imgs/' + remoteVar.getName() + '_' + formatRemoteValue(remoteVar) + '.png';
      valueElement.attr("src", valueContent);                     
    } 
  }
}

function updateOnEnter() {
  var varNames = [];
  for ( var v in mVariablesUIModel ) {
    varNames.push(v);
  }
  if( !mDeviceEngine.getVariablesByName(varNames, deviceEngineResultCallback) ) {
//  alert( 'Cannot start variables read procedure' );
  }
}
       
function startRefresh(secs) {
  setTimeout(function ()
  {
    var varNames = [];
    for ( var v in mVariablesUIModel ) {
      varNames.push(v);
    }
    if( !mDeviceEngine.getVariablesByName(varNames, deviceEngineResultCallback) ) {
//    alert( 'Cannot start variables read procedure' );
      startRefresh(secs);
    }
  }, secs * 1000);
}

function formatRemoteValue(remoteVar) {
  var varDef = mVariablesUIModel[remoteVar.getName()];
  var multiplier = varDef.multiplier || 1;
  var decimals = varDef.decimals;
  if ( typeof(decimals) === "undefined" ) {
    decimals = 2;
  }
  return (remoteVar.getValue() / multiplier).toFixed(decimals);
}

function updateInputFromRemote(remoteVar) {
  var inputElement = getInputUIElementForVar(remoteVar.getName());
  var handleRadio = false;
  if( typeof(inputElement) !== 'undefined' ) {
    inputElement.each(function() {
      var currObj = $(this);
      var ro = mVariablesUIModel[remoteVar.getName()].readonly || '';
      if ( ro === '' )
        currObj.prop('disabled', false);
      else {
        currObj.prop('disabled', true);
      }
      if ( currObj.prop('tagName') === 'SELECT' ) {
        currObj.val(remoteVar.getValue());
      } else if ( currObj.prop('tagName') === 'INPUT' ) {
        var inputType = currObj.prop('type');
        switch ( inputType ) {
          case 'text':
          case 'password':
            if( inputElement.is(":focus") ) {
              // avoid refreshing text items currently "in edit" by the user
              return;
            }
            currObj.val(formatRemoteValue(remoteVar));
            break;
          case 'checkbox':
            currObj.prop('checked', remoteVar.getValue() === '1');
            break;
          case 'radio':
            handleRadio = true;
            break;
        }
      }
    });
    if ( handleRadio ) {
      var radioToCheck = $('[name="' + remoteVar.getName() + '"][value="' + remoteVar.getValue() + '"]');
      radioToCheck.prop('checked', true);
    }
  }
}

function extractVarValue(elem) {
  var value = null;
  if ( elem.prop('tagName') === 'SELECT' ) {
    // for "selects" send the value of the currently selected option
    value = elem.val();
  } else if ( elem.prop('tagName') === 'INPUT' ) {
      if ( elem.prop('type') === 'radio' ) {
        // notify only checked radio buttons since they contain our "selected value"
        if ( elem.prop('checked') ) {
          // radio buttons are special in that they share the name to form a "group"
          value = elem.prop('value');
        }
      } else if ( elem.prop('type') === 'checkbox' ) {
        // checkbox sends 1 if checked, 0 if unchecked
        if ( elem.prop('checked') ) {
          value = '1';
        } else {
          value = '0';
        }
      } else if ( elem.prop('type') === 'text' ) {
        // get raw text from input element (should care about decimal separator?)
        value = elem.val();
    }
  }
  return value;
}

function sendInputValueToRemote(rawElem) {
  var elem = $(rawElem);
  var varname = elem.prop('name');
  var varvalue = extractVarValue(elem);
  commitVarValue(varname, varvalue);
}

function commitVarValue(varname, varvalue) {
  var deviceVariable = findDeviceVariable(varname);
  if ( deviceVariable != null && typeof(deviceVariable) !== 'undefined' ) {
    if ( varvalue != null && typeof(varvalue) !== 'undefined' ) {
      var multiplier = mVariablesUIModel[varname].multiplier || 1;
      deviceVariable.setValue(varvalue * multiplier);
      // do it synchronously to avoid conflicts with asynchronous refreshes
      mDeviceEngine.commitVariables( [ deviceVariable ] );
    }
  }
}
